@@ -54,7 +54,7 @@ demoteHeaders = withTags $ \tag -> case tag of
 
 --------------------------------------------------------------------------------
 isUrlAttribute :: String -> Bool
-isUrlAttribute = (`elem` ["src", "href"])
+isUrlAttribute = (`elem` ["src", "href", "data"])
 
 
 --------------------------------------------------------------------------------
@@ -120,7 +120,7 @@ toSiteRoot = emptyException . joinPath . map parent
 --------------------------------------------------------------------------------
 -- | Check if an URL links to an external HTTP(S) source
 isExternal :: String -> Bool
-isExternal url = any (flip isPrefixOf url) ["http://", "https://"]
+isExternal url = any (flip isPrefixOf url) ["http://", "https://", "//"]
 
 
 --------------------------------------------------------------------------------
