@@ -1,10 +1,11 @@
 --------------------------------------------------------------------------------
+{-# LANGUAGE ExistentialQuantification #-}
 module Hakyll.Web.Template.Context
-    ( Context (..)
-    , mapContext
+    ( ContextField (..)
+    , Context (..)
     , field
     , constField
-    , functionField
+    , listField
 
     , defaultContext
     , bodyField
@@ -17,6 +18,7 @@ module Hakyll.Web.Template.Context
     , getItemUTC
     , modificationTimeField
     , modificationTimeFieldWith
+    , teaserField
     , missingField
     ) where
 
@@ -40,13 +42,20 @@ import           Hakyll.Core.Identifier
 import           Hakyll.Core.Item
 import           Hakyll.Core.Metadata
 import           Hakyll.Core.Provider
-import           Hakyll.Core.Util.String       (splitAll)
+import           Hakyll.Core.Util.String       (splitAll, needlePrefix)
 import           Hakyll.Web.Html
 
 
+--------------------------------------------------------------------------------
+-- | Mostly for internal usage
+data ContextField
+    = StringField String
+    | forall a. ListField (Context a) [Item a]
+
+
 --------------------------------------------------------------------------------
 newtype Context a = Context
-    { unContext :: String -> Item a -> Compiler String
+    { unContext :: String -> Item a -> Compiler ContextField
     }
 
 
@@ -57,13 +66,13 @@ instance Monoid (Context a) where
 
 
 --------------------------------------------------------------------------------
-mapContext :: (String -> String) -> Context a -> Context a
-mapContext f (Context g) = Context $ \k i -> f <$> g k i
+field' :: String -> (Item a -> Compiler ContextField) -> Context a
+field' key value = Context $ \k i -> if k == key then value i else empty
 
 
 --------------------------------------------------------------------------------
 field :: String -> (Item a -> Compiler String) -> Context a
-field key value = Context $ \k i -> if k == key then value i else empty
+field key value = field' key (fmap StringField . value)
 
 
 --------------------------------------------------------------------------------
@@ -72,12 +81,8 @@ constField key = field key . const . return
 
 
 --------------------------------------------------------------------------------
-functionField :: String -> ([String] -> Item a -> Compiler String) -> Context a
-functionField name value = Context $ \k i -> case words k of
-    []              -> empty
-    (n : args)
-        | n == name -> value args i
-        | otherwise -> empty
+listField :: String -> Context a -> Compiler [Item a] -> Context b
+listField key c xs = field' key $ \_ -> fmap (ListField c) xs
 
 
 --------------------------------------------------------------------------------
@@ -91,6 +96,11 @@ defaultContext =
     missingField
 
 
+--------------------------------------------------------------------------------
+teaserSeparator :: String
+teaserSeparator = "<!--more-->"
+
+
 --------------------------------------------------------------------------------
 bodyField :: String -> Context String
 bodyField key = field key $ return . itemBody
@@ -101,7 +111,7 @@ bodyField key = field key $ return . itemBody
 metadataField :: Context String
 metadataField = Context $ \k i -> do
     value <- getMetadataField (itemIdentifier i) k
-    maybe empty return value
+    maybe empty (return . StringField) value
 
 
 --------------------------------------------------------------------------------
@@ -120,7 +130,7 @@ pathField key = field key $ return . toFilePath . itemIdentifier
 --------------------------------------------------------------------------------
 -- | This title field takes the basename of the underlying file by default
 titleField :: String -> Context a
-titleField key = mapContext takeBaseName $ pathField key
+titleField key = field key $ return . takeBaseName . toFilePath . itemIdentifier
 
 
 --------------------------------------------------------------------------------
@@ -203,6 +213,7 @@ getItemUTC locale id' = do
         , "%Y-%m-%d"
         , "%B %e, %Y %l:%M %p"
         , "%B %e, %Y"
+        , "%b %d, %Y"
         ]
 
 
@@ -224,8 +235,20 @@ modificationTimeFieldWith locale key fmt = field key $ \i -> do
     return $ formatTime locale fmt mtime
 
 
+--------------------------------------------------------------------------------
+-- | A context with "teaser" key which contain a teaser of the item.
+-- The item is loaded from the given snapshot (which should be saved
+-- in the user code before any templates are applied).
+teaserField :: String           -- ^ Key to use
+            -> Snapshot         -- ^ Snapshot to load
+            -> Context String   -- ^ Resulting context
+teaserField key snapshot = field key $ \item ->
+    (needlePrefix teaserSeparator . itemBody) <$>
+    loadSnapshot (itemIdentifier item) snapshot
+
+
 --------------------------------------------------------------------------------
 missingField :: Context a
-missingField = Context $ \k i -> compilerThrow $
+missingField = Context $ \k i -> fail $
     "Missing field $" ++ k ++ "$ in context for item " ++
     show (itemIdentifier i)
